@@ -24,12 +24,6 @@ namespace CppEditor {
 
 namespace {
 
-class FriendlyThread: public QThread
-{
-public:
-    using QThread::msleep;
-};
-
 class CollectSymbols: protected SymbolVisitor
 {
     Document::Ptr _doc;
@@ -184,6 +178,12 @@ protected:
         return true;
     }
 
+    bool visit(TemplateTypeArgument *symbol) override
+    {
+        addType(symbol->name());
+        return true;
+    }
+
     bool visit(Enum *symbol) override
     {
         addType(symbol->name());
@@ -270,28 +270,40 @@ static bool acceptName(NameAST *ast, unsigned *referenceToken)
             && !ast->asOperatorFunctionId();
 }
 
-CheckSymbols::Future CheckSymbols::go(Document::Ptr doc, const LookupContext &context, const QList<CheckSymbols::Result> &macroUses)
+CheckSymbols::Future CheckSymbols::go(Document::Ptr doc,
+                                      const QString &content,
+                                      const LookupContext &context,
+                                      const QList<CheckSymbols::Result> &macroUses)
 {
     QTC_ASSERT(doc, return Future());
     QTC_ASSERT(doc->translationUnit(), return Future());
     QTC_ASSERT(doc->translationUnit()->ast(), return Future());
 
-    return (new CheckSymbols(doc, context, macroUses))->start();
+    return (new CheckSymbols(doc, content, context, macroUses))->start();
 }
 
-CheckSymbols * CheckSymbols::create(Document::Ptr doc, const LookupContext &context,
-                                    const QList<CheckSymbols::Result> &macroUses)
+CheckSymbols *CheckSymbols::create(Document::Ptr doc,
+                                   const QString &content,
+                                   const LookupContext &context,
+                                   const QList<CheckSymbols::Result> &macroUses)
 {
     QTC_ASSERT(doc, return nullptr);
     QTC_ASSERT(doc->translationUnit(), return nullptr);
     QTC_ASSERT(doc->translationUnit()->ast(), return nullptr);
 
-    return new CheckSymbols(doc, context, macroUses);
+    return new CheckSymbols(doc, content, context, macroUses);
 }
 
-CheckSymbols::CheckSymbols(Document::Ptr doc, const LookupContext &context, const QList<CheckSymbols::Result> &macroUses)
-    : ASTVisitor(doc->translationUnit()), _doc(doc), _context(context)
-    , _lineOfLastUsage(0), _macroUses(macroUses)
+CheckSymbols::CheckSymbols(Document::Ptr doc,
+                           const QString &content,
+                           const LookupContext &context,
+                           const QList<CheckSymbols::Result> &macroUses)
+    : ASTVisitor(doc->translationUnit())
+    , _doc(doc)
+    , _content(content)
+    , _context(context)
+    , _lineOfLastUsage(0)
+    , _macroUses(macroUses)
 {
     int line = 0;
     getTokenEndPosition(translationUnit()->ast()->lastToken(), &line, nullptr);
@@ -1114,7 +1126,7 @@ bool CheckSymbols::visit(FunctionDefinitionAST *ast)
     accept(ast->ctor_initializer);
     accept(ast->function_body);
 
-    const Internal::LocalSymbols locals(_doc, ast);
+    const Internal::LocalSymbols locals(_doc, _content, ast);
     for (const QList<Result> &uses : std::as_const(locals.uses)) {
         for (const Result &u : uses)
             addUse(u);
@@ -1244,7 +1256,8 @@ bool CheckSymbols::maybeAddTypeOrStatic(const QList<LookupItem> &candidates, Nam
         if (c->isTypedef() || c->asNamespace() ||
                 c->isStatic() || //consider also static variable
                 c->asClass() || c->asEnum() || isTemplateClass(c) ||
-                c->asForwardClassDeclaration() || c->asTypenameArgument() || c->enclosingEnum()) {
+                c->asForwardClassDeclaration() || c->asTypenameArgument() ||
+                c->asTemplateTypeArgument() || c->enclosingEnum()) {
             int line, column;
             getTokenStartPosition(startToken, &line, &column);
             const unsigned length = tok.utf16chars();
