@@ -19,11 +19,11 @@
 #include <utils/utilsicons.h>
 
 #include <QCheckBox>
-#include <QDebug>
 #include <QGroupBox>
 #include <QHBoxLayout>
 #include <QHeaderView>
 #include <QLabel>
+#include <QLoggingCategory>
 #include <QMenu>
 #include <QPointer>
 #include <QScopedPointer>
@@ -36,10 +36,10 @@
 #include <QTreeView>
 #include <QVBoxLayout>
 
+using namespace Core;
 using namespace Utils;
 
-enum { debug = 0 };
-enum { defaultLineWidth = 72 };
+static Q_LOGGING_CATEGORY(log, "qtc.vcs.submiteditor", QtWarningMsg);
 
 /*!
     \class VcsBase::SubmitEditorWidget
@@ -67,6 +67,8 @@ enum { defaultLineWidth = 72 };
 
 namespace VcsBase {
 
+enum { MinSubjectLength = 20, MaxSubjectLength = 72, WarningSubjectLength = 55 };
+
 // QActionPushButton: A push button tied to an action
 // (similar to a QToolButton)
 class QActionPushButton : public QToolButton
@@ -95,15 +97,16 @@ struct SubmitEditorWidgetPrivate
     // A pair of position/action to extend context menus
     typedef QPair<int, QPointer<QAction> > AdditionalContextMenuAction;
 
-    Core::MiniSplitter *splitter;
-    QGroupBox *descriptionBox;
-    QVBoxLayout *descriptionLayout;
-    QLabel *descriptionHint;
-    Utils::CompletingTextEdit *description;
-    QCheckBox *checkAllCheckBox;
-    QTreeView *fileView;
-    QHBoxLayout *buttonLayout;
-    QVBoxLayout *vboxLayout;
+    MiniSplitter *splitter = nullptr;
+    QGroupBox *descriptionBox = nullptr;
+    QVBoxLayout *descriptionLayout = nullptr;
+    QLabel *descriptionHint = nullptr;
+    CompletingTextEdit *description = nullptr;
+    QCheckBox *checkAllCheckBox = nullptr;
+    QTreeView *fileView = nullptr;
+    QHBoxLayout *buttonLayout = nullptr;
+    QLabel *error = nullptr;
+    QVBoxLayout *vboxLayout = nullptr;
 
     QList<AdditionalContextMenuAction> descriptionEditContextMenuActions;
     QVBoxLayout *m_fieldLayout = nullptr;
@@ -112,8 +115,9 @@ struct SubmitEditorWidgetPrivate
     QActionPushButton *m_submitButton = nullptr;
     QString m_description;
     QTimer delayedVerifyDescriptionTimer;
+    int m_delayedVerifyDescriptionInterval = 2000;
 
-    int m_lineWidth = defaultLineWidth;
+    int m_lineWidth = MaxSubjectLength;
     int m_activatedRow = -1;
 
     bool m_filesSelected = false;
@@ -153,7 +157,6 @@ SubmitEditorWidget::SubmitEditorWidget() :
     d->descriptionLayout->addWidget(d->description);
 
     d->delayedVerifyDescriptionTimer.setSingleShot(true);
-    d->delayedVerifyDescriptionTimer.setInterval(500);
     connect(&d->delayedVerifyDescriptionTimer, &QTimer::timeout,
             this, &SubmitEditorWidget::verifyDescription);
 
@@ -175,7 +178,7 @@ SubmitEditorWidget::SubmitEditorWidget() :
     verticalLayout_2->addWidget(d->checkAllCheckBox);
     verticalLayout_2->addWidget(d->fileView);
 
-    d->splitter = new Core::MiniSplitter(scrollAreaWidgetContents);
+    d->splitter = new MiniSplitter(scrollAreaWidgetContents);
     d->splitter->setObjectName("splitter");
     d->splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     d->splitter->setOrientation(Qt::Horizontal);
@@ -184,15 +187,18 @@ SubmitEditorWidget::SubmitEditorWidget() :
 
     d->buttonLayout = new QHBoxLayout();
     d->buttonLayout->setContentsMargins(0, -1, -1, -1);
-    QToolButton *openSettingsButton = new QToolButton;
+    auto openSettingsButton = new QToolButton;
     openSettingsButton->setIcon(Utils::Icons::SETTINGS.icon());
-    openSettingsButton->setToolTip(Core::ICore::msgShowOptionsDialog());
+    openSettingsButton->setToolTip(ICore::msgShowOptionsDialog());
     connect(openSettingsButton, &QToolButton::clicked,  this, [] {
-        Core::ICore::showOptionsDialog(Constants::VCS_COMMON_SETTINGS_ID);
+        ICore::showOptionsDialog(Constants::VCS_COMMON_SETTINGS_ID);
     });
     d->buttonLayout->addWidget(openSettingsButton);
     d->buttonLayout->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
+    d->error = new QLabel();
+    d->buttonLayout->addWidget(d->error);
+
     d->vboxLayout = new QVBoxLayout(scrollAreaWidgetContents);
     d->vboxLayout->setSpacing(6);
     d->vboxLayout->setContentsMargins(9, 9, 9, 9);
@@ -250,14 +256,27 @@ void SubmitEditorWidget::registerActions(QAction *editorUndoAction, QAction *edi
     }
 
     if (submitAction) {
-        if (debug) {
-            const SubmitFileModel *model = fileModel();
-            int count = model ? model->rowCount() : 0;
-            qDebug() << Q_FUNC_INFO << submitAction << count << "items";
-        }
-        d->m_commitEnabled = !canSubmit();
-        connect(this, &SubmitEditorWidget::submitActionEnabledChanged,
-                submitAction, &QAction::setEnabled);
+        auto updateSubmitEnabled = [this, submitAction] {
+            QString errorMessage;
+            const bool submitEnabled = canSubmit(&errorMessage);
+            submitAction->setEnabled(submitEnabled);
+
+            if (submitEnabled || errorMessage.isEmpty()) {
+                d->error->clear();
+            } else {
+                const QString hint = QString("<font color=\"%1\">")
+                    .arg(Utils::creatorColor(Utils::Theme::TextColorError).name());
+                d->error->setText(hint + Tr::tr("Cannot commit: %1").arg(errorMessage));
+            }
+        };
+
+        const SubmitFileModel *model = fileModel();
+        const int itemCount = model ? model->rowCount() : 0;
+        qCDebug(log) << Q_FUNC_INFO << submitAction << itemCount << "items";
+
+        updateSubmitEnabled();
+        connect(this, &SubmitEditorWidget::submitActionEnabledChanged, this, updateSubmitEnabled);
+        connect(this, &SubmitEditorWidget::submitActionTextChanged, this, updateSubmitEnabled);
         connect(this, &SubmitEditorWidget::submitActionTextChanged,
                 submitAction, &QAction::setText);
         d->m_submitButton = new QActionPushButton(submitAction);
@@ -271,8 +290,7 @@ void SubmitEditorWidget::registerActions(QAction *editorUndoAction, QAction *edi
         });
     }
     if (diffAction) {
-        if (debug)
-            qDebug() << diffAction << d->m_filesSelected;
+        qCDebug(log) << diffAction << d->m_filesSelected;
         diffAction->setEnabled(d->m_filesSelected);
         connect(this, &SubmitEditorWidget::fileSelectionChanged, diffAction, &QAction::setEnabled);
         connect(diffAction, &QAction::triggered, this, &SubmitEditorWidget::triggerDiffSelected);
@@ -336,6 +354,12 @@ void SubmitEditorWidget::wrapDescription()
     }
 }
 
+void VcsBase::SubmitEditorWidget::clearDescriptionHint()
+{
+    d->descriptionHint->setText(QString());
+    d->descriptionHint->setToolTip(QString());
+}
+
 QString SubmitEditorWidget::descriptionText() const
 {
     return d->m_description;
@@ -353,8 +377,7 @@ bool SubmitEditorWidget::lineWrap() const
 
 void SubmitEditorWidget::setLineWrap(bool v)
 {
-    if (debug)
-        qDebug() << Q_FUNC_INFO << v;
+    qCDebug(log) << Q_FUNC_INFO << v;
     if (v) {
         d->description->setLineWrapColumnOrWidth(d->m_lineWidth);
         d->description->setLineWrapMode(QTextEdit::FixedColumnWidth);
@@ -371,8 +394,7 @@ int SubmitEditorWidget::lineWrapWidth() const
 
 void SubmitEditorWidget::setLineWrapWidth(int v)
 {
-    if (debug)
-        qDebug() << Q_FUNC_INFO << v << lineWrap();
+    qCDebug(log) << Q_FUNC_INFO << v << lineWrap();
     if (d->m_lineWidth == v)
         return;
     d->m_lineWidth = v;
@@ -391,16 +413,6 @@ void SubmitEditorWidget::setDescriptionMandatory(bool v)
     d->m_descriptionMandatory = v;
 }
 
-QAbstractItemView::SelectionMode SubmitEditorWidget::fileListSelectionMode() const
-{
-    return d->fileView->selectionMode();
-}
-
-void SubmitEditorWidget::setFileListSelectionMode(QAbstractItemView::SelectionMode sm)
-{
-    d->fileView->setSelectionMode(sm);
-}
-
 void SubmitEditorWidget::setFileModel(SubmitFileModel *model)
 {
     d->fileView->clearSelection(); // trigger the change signals
@@ -578,14 +590,12 @@ void SubmitEditorWidget::hideDescription()
 void SubmitEditorWidget::verifyDescription()
 {
     if (!isEnabled()) {
-        d->descriptionHint->setText(QString());
-        d->descriptionHint->setToolTip(QString());
+        clearDescriptionHint();
         return;
     }
 
     auto fontColor = [](Utils::Theme::Color color) {
-        return QString("<font color=\"%1\">")
-                .arg(Utils::creatorTheme()->color(color).name());
+        return QString("<font color=\"%1\">").arg(Utils::creatorColor(color).name());
     };
     const QString hint = fontColor(Utils::Theme::OutputPanes_TestWarnTextColor);
     const QString warning = fontColor(Utils::Theme::TextColorError);
@@ -604,7 +614,6 @@ void SubmitEditorWidget::verifyDescription()
         subjectLength = descriptionLength;
     }
 
-    enum { MinSubjectLength = 20, MaxSubjectLength = 72, WarningSubjectLength = 55 };
     QStringList hints;
     if (0 < subjectLength && subjectLength < MinSubjectLength)
         hints.append(warning + Tr::tr("Warning: The commit subject is very short."));
@@ -635,7 +644,17 @@ void SubmitEditorWidget::verifyDescription()
 void SubmitEditorWidget::descriptionTextChanged()
 {
     d->m_description = cleanupDescription(d->description->toPlainText());
-    d->delayedVerifyDescriptionTimer.start();
+
+    if (d->m_description.isEmpty()) {
+        d->m_delayedVerifyDescriptionInterval = 2000;
+        clearDescriptionHint();
+    } else if (d->m_description.length() > MinSubjectLength || d->m_description.contains("\n")) {
+        d->m_delayedVerifyDescriptionInterval = 100;
+    } else {
+        d->m_delayedVerifyDescriptionInterval = 2000;
+    }
+    d->delayedVerifyDescriptionTimer.start(d->m_delayedVerifyDescriptionInterval);
+
     wrapDescription();
     trimDescription();
     // append field entries
@@ -725,15 +744,10 @@ void SubmitEditorWidget::addDescriptionEditContextMenuAction(QAction *a)
             .push_back(SubmitEditorWidgetPrivate::AdditionalContextMenuAction(-1, a));
 }
 
-void SubmitEditorWidget::insertDescriptionEditContextMenuAction(int pos, QAction *a)
-{
-    d->descriptionEditContextMenuActions
-            .push_back(SubmitEditorWidgetPrivate::AdditionalContextMenuAction(pos, a));
-}
-
 void SubmitEditorWidget::editorCustomContextMenuRequested(const QPoint &pos)
 {
-    QScopedPointer<QMenu> menu(d->description->createStandardContextMenu());
+    QMenu *menu = d->description->createStandardContextMenu();
+    menu->setAttribute(Qt::WA_DeleteOnClose);
     // Extend
     for (const SubmitEditorWidgetPrivate::AdditionalContextMenuAction &a :
          std::as_const(d->descriptionEditContextMenuActions)) {
