@@ -23,7 +23,7 @@ static void print(const QString &fileName, int lineNo, const QString &msg)
 
 class ParseHandler : public QMakeParserHandler {
 public:
-    virtual void message(int /* type */, const QString &msg, const QString &fileName, int lineNo)
+    void message(int /* type */, const QString &msg, const QString &fileName, int lineNo) override
         { print(fileName, lineNo, msg); }
 };
 
